package com.vendor.burn.commands;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import com.vendor.burn.commands.SerialCommand;
import com.vendor.burn.model.FileSystem;
import com.vendor.burn.model.FlashType;
import com.vendor.burn.model.define.IBurner;

public class WriteCommand extends SerialCommand {
  public static final String KEY_SSD = "nvme write";
  
  private static final String KEY_WRITE = "write";
  
  private static final String KEY_WRITE_YAFFS = "write.yaffs";
  
  private static final String KEY_WRITE_YAFFSUC = "write.yaffsuc";
  
  private static final String KEY_MMC = "mmc write";
  
  private static final String KEY_MMC_BOOTWRITE = "mmc bootwrite";
  
  private static final String KEY_MMC_SP = "mmc write.ext4sp";
  
  private static final String KEY_SSD_SP = "nvme write.sparse";
  
  private static final String KEY_UFS = "ufs write";
  
  private static final String KEY_UFS_SP = "ufs write.ext4sp";
  
  private static final String KEY_UFS1 = "ufs bootwrite";
  
  private static final int HEX = 16;
  
  public static final int EMMC_ALIGN_LEN = 512;
  
  public static final int UFS_ALIGN_LEN = 4096;
  
  private WriteCommand(String command, String length) {
    super(command);
    setTimeout(calculateTimeOut(length));
  }
  
  private static com.vendor.burn.commands.WriteCommand write(FlashType flashType, String key, String destination, String orignal, String length) {
    StringBuilder sb = new StringBuilder();
    if (flashType == FlashType.spi) {
      sb.append("sf");
    } else {
      sb.append(flashType.name());
    } 
    sb.append(" ").append(key).append(" ");
    if (!destination.startsWith("0x"))
      sb.append("0x"); 
    sb.append(destination).append(" ");
    if (!orignal.startsWith("0x"))
      sb.append("0x"); 
    sb.append(orignal).append(" ");
    if (!length.startsWith("0x"))
      sb.append("0x"); 
    sb.append(length);
    return new com.vendor.burn.commands.WriteCommand(sb.toString(), length);
  }
  
  public static com.vendor.burn.commands.WriteCommand writeeMMC(FlashType flashType, String address, String startSector, String lengthSector) {
    if (flashType == FlashType.ufs1)
      return getUfsCmd("ufs bootwrite", address, startSector, lengthSector); 
    if (FlashType.fuzzyCompareFlash(flashType, FlashType.ufs))
      return getUfsCmd("ufs write", address, startSector, lengthSector); 
    StringBuilder sb = new StringBuilder();
    if (flashType == FlashType.ssd) {
      sb.append("nvme write").append(" ");
    } else {
      sb.append("mmc write");
      sb.append(" ").append("0x0").append(" ");
    } 
    if (!address.startsWith("0x"))
      sb.append("0x"); 
    sb.append(address).append(" ");
    if (!startSector.startsWith("0x"))
      sb.append("0x"); 
    sb.append(startSector).append(" ");
    if (!lengthSector.startsWith("0x"))
      sb.append("0x"); 
    sb.append(lengthSector);
    return (com.vendor.burn.commands.WriteCommand)new Object(sb.toString(), lengthSector);
  }
  
  public static com.vendor.burn.commands.WriteCommand writeEmmcOrUfs(FlashType flashType, String address, String startSector, String lengthSector, IBurner burner) {
    if (flashType == FlashType.ufs1)
      return getUfsCmd("ufs bootwrite", address, startSector, lengthSector); 
    return writeeMMC(flashType, address, startSector, lengthSector);
  }
  
  private static com.vendor.burn.commands.WriteCommand getUfsCmd(String firstKey, String address, String startSector, String lengthSector) {
    StringBuilder sb = new StringBuilder(firstKey);
    sb.append(" ").append("0x0").append(" ");
    if (!address.startsWith("0x"))
      sb.append("0x"); 
    sb.append(address).append(" ");
    try {
      if (startSector.startsWith("0x"))
        startSector = startSector.substring("0x".length()); 
      if (lengthSector.startsWith("0x"))
        lengthSector = lengthSector.substring("0x".length()); 
      long realStart = (Long.parseLong(startSector, 16) * 512L + 4096L - 1L) / 4096L;
      long realLength = (Long.parseLong(lengthSector, 16) * 512L + 4096L - 1L) / 4096L;
      String newStartSector = Long.toHexString(realStart);
      String newLengthSector = Long.toHexString(realLength);
      if (!newStartSector.startsWith("0x"))
        sb.append("0x"); 
      sb.append(newStartSector).append(" ");
      if (!newLengthSector.startsWith("0x"))
        sb.append("0x"); 
      sb.append(newLengthSector);
      return (com.vendor.burn.commands.WriteCommand)new Object(sb.toString(), newLengthSector);
    } catch (NumberFormatException e) {
      return null;
    } 
  }
  
  public static com.vendor.burn.commands.WriteCommand writeeMMCSP(FileSystem fileSystem, FlashType flashType, String address, String startSector, String lengthSector) {
    if (FlashType.fuzzyCompareFlash(flashType, FlashType.ufs))
      return getUfsCmd("ufs write.ext4sp", address, startSector, lengthSector); 
    StringBuilder sb = new StringBuilder();
    if (flashType == FlashType.ssd) {
      sb.append("nvme write.sparse");
      sb.append(" ");
    } else {
      sb.append("mmc write.ext4sp");
      sb.append(" ").append("0x0").append(" ");
    } 
    if (!address.startsWith("0x"))
      sb.append("0x"); 
    sb.append(address).append(" ");
    if (!startSector.startsWith("0x"))
      sb.append("0x"); 
    sb.append(startSector).append(" ");
    if (!lengthSector.startsWith("0x"))
      sb.append("0x"); 
    sb.append(lengthSector);
    return (com.vendor.burn.commands.WriteCommand)new Object(sb.toString(), lengthSector);
  }
  
  public static com.vendor.burn.commands.WriteCommand bootWriteeMMC(String address, String sector, String length) {
    StringBuilder sb = new StringBuilder("mmc bootwrite");
    sb.append(" ").append("0x0").append(" ");
    if (!address.startsWith("0x"))
      sb.append("0x"); 
    sb.append(address).append(" ");
    if (!sector.startsWith("0x"))
      sb.append("0x"); 
    sb.append(sector).append(" ");
    if (!length.startsWith("0x"))
      sb.append("0x"); 
    sb.append(length);
    return (com.vendor.burn.commands.WriteCommand)new Object(sb.toString(), length);
  }
  
  public static com.vendor.burn.commands.WriteCommand writeYaffs(FlashType flashType, String destination, String original, String length) {
    return write(flashType, "write.yaffs", destination, original, length);
  }
  
  public static com.vendor.burn.commands.WriteCommand writeYaffsuc(FlashType flashType, String destination, String original, String length) {
    return write(flashType, "write.yaffsuc", destination, original, length);
  }
  
  public static com.vendor.burn.commands.WriteCommand write(FlashType flashType, String destination, String original, String length) {
    return write(flashType, "write", destination, original, length);
  }
}
